#!/usr/bin/gjs-console

const Gio = imports.gi.Gio;
const Gdk = imports.gi.Gdk;
const Gtk = imports.gi.Gtk;
const Prefs = imports.prefs;

function PrefsWidget() {
    this._init.apply(this);
}

PrefsWidget.prototype = {
    _init: function () {
        this.application = new Gtk.Application({
            application_id: 'org.gnome.shell.extensions.prefs.widget',
            flags: Gio.ApplicationFlags.FLAGS_NONE
        });

        this.application.connect('activate', Lang.bind(this, this._onActivate));
        this.application.connect('startup', Lang.bind(this, this._onStartup));
    },

    _onActivate: function () {
        this._window.present();
    },

    _onStartup: function () {
        this._buildUI();
    },

    _buildUI: function () {
        this._window = new Gtk.Window({
            application: this.application,
            window_position: Gtk.WindowPosition.CENTER,
            title: "Prefs widget",
            default_height: 400,
            default_width: 600,
            screen: Gdk.Screen.get_default()
        });
        let vbox = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL });
        this._window.add(vbox);

        this._extensionPrefsBin = new Gtk.Frame();
        vbox.add(this._extensionPrefsBin);

        Prefs.init();
        let widget;
        try {
            widget = Prefs.buildPrefsWidget(); 
        } catch (e) {
            widget = this._buildErrorUI(e);
        }

        // Destroy the current prefs widget, if it exists
        if (this._extensionPrefsBin.get_child())
            this._extensionPrefsBin.get_child().destroy();

        this._extensionPrefsBin.add(widget);

        this._window.show_all();
    },

    _buildErrorUI: function(exc) {
        let box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL });
        let label = new Gtk.Label({
            label: _("There was an error loading the preferences dialog")
        });
        box.add(label);

        let errortext = '';
        errortext += exc;
        errortext += '\n\n';
        errortext += 'Stack trace:\n';

        // Indent stack trace.
        errortext += exc.stack.split('\n').map(function(line) {
            return '  ' + line;
        }).join('\n');

        let scroll = new Gtk.ScrolledWindow({ vexpand: true });
        let buffer = new Gtk.TextBuffer({ text: errortext });
        let textview = new Gtk.TextView({ buffer: buffer });
        textview.override_font(Pango.font_description_from_string('monospace'));
        scroll.add(textview);
        box.add(scroll);

        box.show_all();
        return box;
    }
}

Gtk.init(null);
let app = new PrefsWidget();
app.application.run(ARGV);
